var frames = [];

var innerHeight = null;
var clientHeight = null;
var clientWidth = null;
var tabId = null;
var cPosition = 0;
var initialScroll = 0;
var toType = 'pdf';
var url = 'XX';
var devicePixelRatio = window.devicePixelRatio || 1;


if (navigator.userAgent.toLowerCase().indexOf('firefox') > -1) {
    devicePixelRatio = 1;
}

var selectedSize = false;

var tabHelper = chrome.tabs || browser.tabs;

chrome.runtime.onMessage.addListener(function (message, sender, sendResponse) {

    switch (message.event) {
        case 'doScreenShotFull':
            init();

            tabId = message.currentTab.id;
            toType = message.toType;
            url = message.url;
            doScreenshotFull();
            break;

        case 'doScreenShotSelection':
            init();
            tabId = message.currentTab.id;
            toType = message.toType;
            url = message.url;
            doScreenshotSelection();
            break;

        case 'doScreenShotViewport':
            init();
            tabId = message.currentTab.id;
            toType = message.toType;
            url = message.url;
            doScreenshotViewportSingle();
            break;

        case 'receiveSelection':
            selectedSize = message.data;

            doScreenshotFull();

            break;
    }
});

function init() {
    frames = [];

    innerHeight = null;
    clientHeight = null;
    clientWidth = null;

    tabId = null;

    cPosition = 0;

    initialScroll = 0;

    selectedSize = false;
}

function doScreenshotViewPort(doAfterScreenshot) {
    tabHelper.query({active: true, lastFocusedWindow: true}, function (tabs) {
        tabHelper.captureVisibleTab(null, {format: 'png'}, function (data) {
            frames.push(data);
            if (typeof doAfterScreenshot !== 'undefined') {
                doAfterScreenshot();
            }
        });
    });
}

function doScreenshotSelection() {
    tabHelper.sendMessage(tabId, {text: 'getSize'}, function (sizes) {
        gotSize(sizes, initCanvas);
    });
}

function initCanvas() {
    tabHelper.sendMessage(tabId, {text: 'initCanvas'});
}

function doScreenshotFull() {
    tabHelper.sendMessage(tabId, {text: 'getSize'}, function (sizes) {
        gotSize(sizes, scrollScreenshot);
    });
}

function doScreenshotViewportSingle() {
    tabHelper.sendMessage(tabId, {text: 'getSize'}, function (sizes) {
        gotSize(sizes, function () {
            doScreenshotViewPort(function () {
                submitImage(frames[0]);
            });
        });
    });
}

function gotSize(sizes, callback) {
    innerHeight = sizes.innerHeight;
    clientHeight = sizes.clientHeight;
    clientWidth = sizes.clientWidth;
    initialScroll = sizes.scrollY;

    callback();
}

function scrollScreenshot() {
    if (cPosition <= clientHeight) {
        tabHelper.sendMessage(tabId, {text: 'scrollTo', offset: cPosition, doHide: true}, function () {
            setTimeout(function () {
                doScreenshotViewPort(function () {
                    scrollScreenshot();
                });
                cPosition += innerHeight;
            }, 250);
        });
    } else {
        tabHelper.sendMessage(tabId, {text: 'showHidden'});
        tabHelper.sendMessage(tabId, {text: 'scrollTo', offset: initialScroll, doHide: false});
        joinParts();
    }

}

function joinParts() {
    var canvas = window.document.createElement('canvas');

    canvas.setAttribute('width', clientWidth * devicePixelRatio);
    canvas.setAttribute('height', clientHeight * devicePixelRatio);
    var context = canvas.getContext('2d');

    var heightSumDiff = 0;

    var framePositions = [];

    for (var frame = 0; frame < frames.length; frame++) {
        var img = new Image();
        img.src = frames[frame];
        img.frame = frame;
        img.onload = function () {

            var imgHeight = this.height;

            position = this.frame > 0 ? framePositions[this.frame - 1] + imgHeight : 0;

            heightSumDiff += innerHeight * devicePixelRatio - imgHeight;

            framePositions[this.frame] = position;

            if (frames.length - 1 === this.frame) {
                position = clientHeight * devicePixelRatio - imgHeight - heightSumDiff - devicePixelRatio;
            }

            context.drawImage(this, 0, position);

            if (this.frame == frames.length - 1) {
                setTimeout(function () {
                    data = canvas.toDataURL('image/png');

//                    submitImage(data);

                    if (selectedSize) {
                        cropImage(canvas, selectedSize.x * devicePixelRatio, selectedSize.y * devicePixelRatio, selectedSize.width * devicePixelRatio, selectedSize.height * devicePixelRatio,
                                function (img) {
                                    submitImage(img);
                                });
                    } else {
                        submitImage(data);
                    }
                }, 500);
            }
        }
    }
}

function submitImage(img) {
    if (img.length < 1048576 && toType == 'png'  && navigator.userAgent.toLowerCase().indexOf('firefox') === -1) {

        var link = document.createElement("a");
        link.download = 'website-screenshot.png';
        link.href = img;
        document.body.appendChild(link);
        link.click();
        document.body.removeChild(link);
    } else {

        var data = new FormData();
        data.append('data', img);
        data.append('url', url);

        var xhr = new XMLHttpRequest();
        xhr.open('POST', 'https://online-umwandeln.de/html.php?type=' + toType, true);
        var opened = false;

        xhr.onreadystatechange = function ()
        {
            if (xhr.responseURL && !opened)
            {
                url = xhr.responseURL + '&utm_source=extension&utm_medium=website-in-pdf';
                tabHelper.create({url: url});
                opened = true;
            }
        }
        xhr.send(data);
    }
}


function cropImage(canvas, offsetX, offsetY, width, height, callback) {
    // create an in-memory canvas
    var buffer = document.createElement('canvas');
    var bCtx = buffer.getContext('2d');
    // set its width/height to the required ones
    buffer.width = width;
    buffer.height = height;
    // draw the main canvas on our buffer one
    // drawImage(source, source_X, source_Y, source_Width, source_Height, 
    //  dest_X, dest_Y, dest_Width, dest_Height)
    bCtx.drawImage(canvas, offsetX, offsetY, width, height,
            0, 0, buffer.width, buffer.height);
    // now call the callback with the dataURL of our buffer canvas
    callback(buffer.toDataURL());
}
