var hiddenElements = [];

var initialOverflow = false;

var div, dimDiv;

var height = 0;

chrome.runtime.onMessage.addListener(function (msg, sender, sendResponse) {
    switch (msg.text) {
        case 'getSize':
            var body = document.body;
            var html = document.documentElement;

            height = Math.max(body.scrollHeight, body.offsetHeight,
                    html.clientHeight, html.scrollHeight, html.offsetHeight);
            sendResponse(
                    {
                        innerHeight: window.innerHeight,
                        clientHeight: height, // clientHeight?
                        clientWidth: document.body.clientWidth,
                        scrollY: window.scrollY
                    }
            );
            break;
        case 'scrollTo':
            window.scrollTo(0, msg.offset);

            if (initialOverflow === false) {
                initialOverflow = document.body.style.getPropertyValue('overflow');
                document.body.style.setProperty('overflow', 'hidden');
            }

            if (msg.offset > 0 && hiddenElements.length == 0 && msg.doHide) {
                var iterator = document.createNodeIterator(document.documentElement, NodeFilter.SHOW_ELEMENT, null, false), current;
                while (current = iterator.nextNode())
                {
                    var styles = document.defaultView.getComputedStyle(current, '');
                    if (!styles || styles.getPropertyValue('display') === 'none') {
                        continue;
                    }
                    if (styles.getPropertyValue('position') === 'fixed' || styles.getPropertyValue('position') === 'sticky')
                    {
                        hiddenElements.push({element: current, opacity: styles.getPropertyValue('opacity')});
                        current.style.setProperty('opacity', 0);
                    }
                }
            }

            break;
        case 'showHidden':
            document.body.style.setProperty('overflow', initialOverflow);
            for (i = 0; i < hiddenElements.length; i++) {
                hiddenElements[i].element.style.setProperty('opacity', hiddenElements[i].opacity);
            }
            break;

        case 'initCanvas':
            initCanvas();
            break;
    }

});


function initCanvas() {
    div = document.createElement('div');

    div.style.position = 'absolute';
    div.style.width = document.body.scrollWidth + 'px';
    div.style.height = height + 'px';

    div.style.top = 0;
    div.style.left = 0;

    div.style.backgroundColor = '#eee';
    div.style.opacity = 0.8;

//    div.zIndex = 2147483647;
    div.style.zIndex = 7000;

    document.body.appendChild(div);

    initDraw(div);
}



function initDraw(canvas) {
    function setMousePosition(e) {
        if (e.pageX) {
            mouse.x = e.pageX + window.pageXOffset;
            mouse.y = e.pageY + window.pageYOffset;
        }
    }

    var mouse = {
        x: 0,
        y: 0,
        startX: 0,
        startY: 0
    };
    var element = null;

    canvas.onmousemove = function (e) {
        setMousePosition(e);

        if (element !== null) {
            element.style.width = Math.abs(mouse.x - mouse.startX) + 'px';
            element.style.height = Math.abs(mouse.y - mouse.startY - window.scrollY) + 'px';
            element.style.left = (mouse.x - mouse.startX < 0) ? mouse.x + 'px' : mouse.startX + 'px';
            element.style.top = (mouse.y - window.scrollY - mouse.startY < 0) ? mouse.y - window.scrollY + 'px' : mouse.startY + 'px';
        }

    };

    canvas.onmousedown = function (e) {

        if (element !== null) {
            if (parseInt(element.style.width) > 0 && parseInt(element.style.height) > 0) {
                var data = {
                    x: parseInt(element.style.left),
                    y: parseInt(element.style.top),
                    width: parseInt(element.style.width),
                    height: parseInt(element.style.height)
                };

                chrome.runtime.sendMessage({event: 'receiveSelection', data: data});

                document.body.removeChild(div);
            }

            element = null;
            canvas.style.cursor = 'default';

        } else {
            mouse.startX = mouse.x;
            mouse.startY = mouse.y - window.scrollY;
            element = document.createElement('div');

            element.style.left = mouse.x + 'px';
            element.style.top = mouse.y - window.scrollY + 'px';

            dimDiv = document.createElement('span');

            dimDiv.style.position = 'absolute';
            dimDiv.style.bottom = 0;
            dimDiv.style.right = 0;

            element.appendChild(dimDiv);

            canvas.appendChild(element)
            canvas.style.cursor = 'crosshair';

            element.style.border = '1px dashed #ccc';
            element.style.backgroundColor = '#ddd';
            element.style.position = 'absolute';
        }
    };
}